/*
 *
 * Checks for tty accessability and bandwidth saturation.
 *
 * $Header: ttychk.c,v 5.2 87/12/09 16:57:02 kenj Exp $
 */

#include <stdio.h>
#ifdef SysV
#include <termio.h>
#else
#include <sgtty.h>
#endif
#include <signal.h>

struct {
	int	baud;	/* baud rate */
	int	symb;	/* symbolic constant for baud rate */
} bmap[] = {
	{  300,  B300 },
	{ 1200, B1200 },
	{ 2400, B2400 },
	{ 4800, B4800 },
	{ 9600, B9600 },
	{    0,     0 }
};
int		status = 0;	/* exit() status */

main(argc, argv)
int	argc;
char	*argv[];
{
#ifdef SysV
	struct termio	ttyb;
#else
	struct sgttyb	ttyb;
#endif
	int		nusers;		/* number of simulated users */
	int		need;		/* max ttys that could be used */
	int		baud = 0;	/* aggregate baud rate across
					   active ttys */
	int		br;
	int		nttys = 0;	/* number of required ttys */
	int		i;
	int		fd;
	int		l;
	float		orate;		/* estimated output rate per user */
        int		onalarm();

	if (argc < 4) {
		fprintf(stderr, "Usage: ttychk nusers orate ttydev ... \n");
		exit(1);
	}
	nusers = atoi(argv[1]);
	if (nusers < 1) {
		fprintf(stderr, "ttychk: nusers must be > 0\n");
		status |= 1;
	}
	sscanf(argv[2], "%f", &orate);

	need = nusers;

	signal(SIGALRM, onalarm);
	for (i = 3; i < argc && need; i++, nttys++, need--) {
		alarm(3);
		if ((fd = open(argv[i], 1)) < 0) {
			fprintf(stderr, "ttychk: cannot open %s for writing\n",
				argv[i]);
			perror("");
			status |= 1;
			continue;
		}
		alarm(0);
#ifdef SysV
		if (ioctl(fd, TCGETA, &ttyb) < 0) {
#else
		if (ioctl(fd, TIOCGETP, &ttyb) < 0) {
#endif
			fprintf(stderr, "ttychk: cannot stat %s\n",
				argv[i]);
			perror("");
			status |= 1;
			close(fd);
			continue;
		}
		close(fd);
		br = 0;
		for (l = 0; bmap[l].baud; l++) {
#ifdef SysV
			if (bmap[l].symb == (ttyb.c_cflag & CBAUD)) {
#else
			if (bmap[l].symb == ttyb.sg_ospeed) {
#endif
				br = bmap[l].baud;
				break;
			}
		}
#ifdef DEBUG
		fprintf(stderr, "%s: %d baud\n", argv[i], br);
#endif
		if (br != 9600)
			fprintf(stderr, "ttychk: Warning - baud rate at %d for %s\n",
				br, argv[i]);
		baud += br;
	}
#ifdef DEBUG
	fprintf(stderr, "aggregate tty bandwidth: %d baud\n", baud);
	fprintf(stderr, "aggregate tty output rate: %d chars/sec (for %d ttys)\n", (int)(orate*nusers), nttys);
#endif

	/* assume 10 bits per character, and the 50% figure is arbitrary! */
	if (nusers*orate > 0.5*(baud/10)) {
		fprintf(stderr, "ttychk panic: Total tty output rate (%d chars/sec) exceeds 50%% of\n              total tty bandwid th (%d chars/sec)\n",
			(int)(nusers*orate), baud/10);
		status |= 1;
	}

	exit(status);
}

onalarm()
{
	status |= 1;
}


